/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2003
 *
 *    File name      : hid_mouse.c
 *    Description    : Define HID module
 *
 *    History :
 *    1. Data        : December 19, 2005
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: 1.3 $
**************************************************************************/
#include "hid_mouse.h"

#pragma data_alignment=4
Int8U DataBuffer[sizeof(UsbHidDescriptor_t)];
#pragma data_alignment=4
MouseReport_t MouseReport = {0,0,0};


/*************************************************************************
 * Function Name: HidInit
 * Parameters: none
 *
 * Return: none
 *
 * Description: Init HID Mouse
 *
 *************************************************************************/
void HidInit (void)
{
  MouseReport.Buttons = 0;
  MouseReport.X = 0;
  MouseReport.Y = 0;
  UsbCoreInit();
}

/*************************************************************************
 * Function Name: UsbClassHidDescriptor
 * Parameters:  pUsbSetupPacket_t pSetup
 *
 * Return: UsbCommStatus_t
 *
 * Description: Implement GET DESCRIPTOR
 *
 *************************************************************************/
UsbCommStatus_t UsbClassHidDescriptor (pUsbSetupPacket_t pSetup)
{
  if (pSetup->wIndex.Word == HID_INTERFACE_0)
  {
    switch (pSetup->wValue.Hi)
    {
    case Hid:
      USB_IO_Data(CTRL_ENP_IN,
                (pInt8U)UsbHidDescriptor,
                USB_T9_Size(SIZE_OF_HID_DESC,pSetup->wLength.Word),
                (void *)USB_StatusOutHandler);
      return(UsbPass);
    case HidReport:
      USB_IO_Data(CTRL_ENP_IN,
                 (pInt8U)mouseDescriptor,
                 USB_T9_Size(SIZE_OF_HID_MOUSE_DESC,pSetup->wLength.Word),
                 (void *)USB_StatusOutHandler);
      return(UsbPass);
    }
  }
  return(UsbNotSupport);
}

/*************************************************************************
 * Function Name: UsbClassHidRequest
 * Parameters:  pUsbSetupPacket_t pSetup
 *
 * Return: UsbCommStatus_t
 *
 * Description: Implement USB Class Hid requests
 *
 *************************************************************************/
UsbCommStatus_t UsbClassHidRequest (pUsbSetupPacket_t pSetup)
{
  switch (pSetup->bRequest)
  {
  case HID_GET_REPORT:
    if((pSetup->wValue.Hi == 1) &&
       (pSetup->wIndex.Word == HID_INTERFACE_0))
    {
      USB_IO_Data(CTRL_ENP_IN,
                 (pInt8U)&MouseReport,
                 USB_T9_Size(ReportEpMaxSize,pSetup->wLength.Word),
                 (void *)USB_StatusOutHandler);
      return(UsbPass);
    }
    break;
  }
  return(UsbNotSupport);
}

/*************************************************************************
 * Function Name: HidSendReport
 * Parameters:  Int8U Buttons, Int8S X Int8S Y
 *
 * Return: none
 *
 * Description: USB HID report send
 *
 *************************************************************************/
void HidSendReport (Int8U Buttons, Int8S X, Int8S Y)
{
  MouseReport.Buttons = Buttons;
  MouseReport.X = X;
  MouseReport.Y = Y;
  USB_IO_Data(ReportEp,
             (pInt8U)&MouseReport,
             ReportEpMaxSize,
             NULL);
}
